TESTVALUES = dict(
    solr_url='http://solr.foo',
    prefix='21.14foo',
    drs_id1='my/drs/id',
    version_number1=201603,
    suffix1='afd65cd0-9296-35bc-a706-be98665c9c36',
    data_node='esgf-foo.dkrz.de',
    url_rabbit_trusted='http://trusted.rabbit.foo',
    url_rabbit_open='http://rabbit.foo',
    rabbit_exchange_name='exch', # TODO JUST ONE NAME!
    thredds_service_path='test/thr/path',
    rabbit_username_open='RogerRabbit',
    rabbit_username_trusted='RogerRabbit',
    rabbit_password='carrotDreams',
    errata_ids=['123456','654321'],
    errata_id='123456',
    file_name1='mytestfilename.nc',
    file_suffix1='xyz123abc',
    file_suffix2='xyz123def',
    checksum1='testchecksumabc',
    file_size1=10010,
    publish_path1='my/publish/path',
    checksum_type1='SHA999',
    file_version1='987'
)

